from enum import Enum


class ApiAutomationListTypeItem(str, Enum):
    DESCRIPTION_REPLACE = "DESCRIPTION_REPLACE"
    FOOD_ALIAS = "FOOD_ALIAS"
    FOOD_REPLACE = "FOOD_REPLACE"
    INSTRUCTION_REPLACE = "INSTRUCTION_REPLACE"
    KEYWORD_ALIAS = "KEYWORD_ALIAS"
    NAME_REPLACE = "NAME_REPLACE"
    NEVER_UNIT = "NEVER_UNIT"
    TRANSPOSE_WORDS = "TRANSPOSE_WORDS"
    UNIT_ALIAS = "UNIT_ALIAS"
    UNIT_REPLACE = "UNIT_REPLACE"

    def __str__(self) -> str:
        return str(self.value)
